from tqdm import tqdm

def bio2bmes(read_file, save_file, split_symbol):
    f = open(read_file, encoding='utf-8')
    f1 = open(save_file, 'w+', encoding='utf_8')

    sentences = []
    sentence = []
    label_set = set()
    cnt_line = 0
    for line in f:

        cnt_line += 1
        if len(line) == 0 or line[0] == "\n":
            if len(sentence) > 0:
                sentences.append(sentence)
                # print(sentence)
                sentence = []
            continue
        splits = line.split(split_symbol)
        sentence.append([splits[0], splits[-1][:-1]])
        label_set.add(splits[-1])

    if len(sentence) > 0:
        sentences.append(sentence)
        sentence = []
    f.close()

    # 文件转换 存储文件
    for sen in sentences:
        i = 0
        for index, word in enumerate(sen):
            char = word[0]
            label = word[1]
            if index < len(sen) - 1:
                if (label[0] == 'B'):
                    if sen[index + 1][1][0] == 'I':
                        label = label
                    elif sen[index + 1][1][0] == 'O':
                        label = 'S' + label[1:]
                elif (label[0] == 'I'):
                    if sen[index + 1][1][0] == 'I':
                        label = 'M' + label[1:]
                    if sen[index + 1][1][0] == 'O' or sen[index + 1][1][0] == 'B':
                        label = 'E' + label[1:]
                elif (label[0] == 'O'):
                    label = label
            else:
                if (label[0] == 'B'):
                    label = 'S' + label[1:]
                elif (label[0] == 'I'):
                    label = 'E' + label[1:]
                elif (label[0] == 'O'):
                    label = label

            f1.write(f'{char} {label}\n')
        f1.write('\n')
    f1.close()

def split_trans(read_file, save_file, split_before, split_after):
    with open(read_file, 'r', encoding='utf8') as f1, open(
            save_file, "w", encoding="utf8") as f2:
        for line in tqdm(f1.readlines()):
            fline = line.split(split_before)
            f2.write(split_after.join(fline))


from functools import wraps

class info_decoratoer:
    def __init__(self):
        pass

def fileOperateInfo(func):
    @wraps(func)
    def info_start_end(*args, **kwargs):
        print("-----开始操作文件-----")
        func(*args, **kwargs)
        print("-----结束操作文件-----")
    return info_start_end


import pandas as pd
import xlsxwriter
from tqdm import tqdm
from typing import List, Dict

from qrutils.decorators.info_decorator import fileOperateInfo

# @fileOperateInfo
def read_xlsx_as_list(read_file: str, need_title=False) -> List[List]:
    all_datas = []
    df = pd.read_excel(read_file)
    titles = df.columns.tolist()
    length = len(df)
    for row in range(length):
        all_datas.append(df.iloc[row].values.tolist())
    if need_title:
        return titles, all_datas
    else:
        return all_datas

# @fileOperateInfo
def read_xlsx_as_dict(read_file: str) -> List[Dict]:
    all_datas = []
    df = pd.read_excel(read_file)
    titles = df.columns.tolist()
    length = len(df)
    for row in range(length):
        tmp_dic = {}
        datas = df.iloc[row]
        for index, data in enumerate(datas):
            tmp_dic[titles[index]] = data
        all_datas.append(tmp_dic)
    return all_datas

# @fileOperateInfo
def write_xlsx_from_list(titles: List[str], datas: List[List[str]], save_file: str) -> None:
    tqdm.write("-----开始写入xlsx-----")
    wb = xlsxwriter.Workbook(save_file)
    ws = wb.add_worksheet()
    for pos, title in enumerate(titles):
        ws.write(0, pos, title)
    for row, data in enumerate(datas):
        for col, dat in enumerate(data):
            ws.write(row+1, col, dat)
    wb.close()

# @fileOperateInfo
def write_xlsx_from_dict(datas: List[Dict], save_file: str) -> None:
    tqdm.write("-----开始写入xlsx-----")
    wb = xlsxwriter.Workbook(save_file)
    ws = wb.add_worksheet()
    titles = datas[0].keys()
    for pos, title in enumerate(titles):
        ws.write(0, pos, title)
    for row, data in enumerate(datas):
        for col, item in enumerate(data.items()):
            ws.write(row+1, col, item[1])
    wb.close()



import json
from tqdm import tqdm
from typing import List, Dict

from qrutils.decorators.info_decorator import fileOperateInfo

@fileOperateInfo
def read_json(read_file: str) -> List[Dict]:
    with open(read_file, 'r', encoding='utf8') as f:
        tmp = json.load(f)
    return tmp

# @fileOperateInfo
def read_datas(read_file):
    return_list = []
    with open(read_file, "r", encoding="utf8") as f:
        for line in f.readlines():
            return_list.append(json.loads(line))
    return return_list

# @fileOperateInfo
def read_datas_with_keys(read_file, spacial_keys):
    return_list = []
    with open(read_file, "r", encoding="utf8") as f:
        for line in f.readlines():
            tmp = json.loads(line)
            dic = {}
            for k in spacial_keys:
                dic[k] = tmp[k]
            return_list.append(dic)
    return return_list

# @fileOperateInfo
def write_jsonl(datas, save_file):
    with open(save_file, "w", encoding="utf8") as f:
        for data in tqdm(datas):
            f.write(json.dumps(data, ensure_ascii=False))
            f.write("\n")

# @fileOperateInfo
def modify_keys(source_data: List[Dict], taget_keys: List) -> List[Dict]:
    return_list = []
    source_keys = list(source_data[0].keys())
    length = len(source_data[0])
    for data in tqdm(source_data):
        tmp_dic = {}
        for pos in range(length):
            tmp_dic[taget_keys[pos]] = data[source_keys[pos]]
        return_list.append(tmp_dic)
    return return_list



from ensurepip import version
from setuptools import find_packages, setup

setup(
    name="qrutils",
    version="0.0.2-beta1",
    author='VictorT',
    packages=find_packages(),
    install_requires=[
        'tqdm',
        'pandas',
    ],
    classifiers=[
        'Programming Language :: Python :: 3.8',
    ]
)



import os

def find_file_in_dir(dir: str, spacial_key=''):
    file_names = []
    for root, dirs ,files in os.walk(dir, topdown=False):
        if spacial_key == '':
            file_names += files
        else:
            for name in files:
                if spacial_key in name:
                    file_names.append(name)
    return file_names


if __name__ == '__main__':
    result = find_file_in_dir('dist', 'whl')
    print(result)



from tqdm import tqdm

def check_conll(file_name, cut_symbol) -> bool:
        with open(file_name, 'r', encoding='utf8') as f:
            for num, line in enumerate(f.readlines()):
                if len(line.strip().split(cut_symbol)) != 2 and line.strip() != "":
                    print("错误行数：", num, "错误行内容：", line)




from tqdm import tqdm

from .Loader import Loader

class ConllLoader(Loader):

    def __init__(self) -> None:
        super().__init__()
        self.texts = []
        self.labels = []

    def load(self, file_name: str) -> None:
        texts = []
        labels = []
        with open(file_name, 'r', encoding='utf8') as f:
            simple_text = []
            simple_label = []
            for line in tqdm(f.readlines()):
                if line.strip() == "":
                    if simple_label != [] and simple_text != []:
                        assert len(simple_text) == len(simple_label)
                        texts.append(simple_text)
                        labels.append(simple_label)
                        simple_text = []
                        simple_label = []
                else:
                    token, tag = line.strip().split("\t")
                    simple_text.append(token)
                    simple_label.append(tag)
            assert len(texts) == len(labels)
        
        self.texts += texts
        self.labels += labels

